<?php

namespace Drupal\thbl\Query;

use Drupal\Core\Database\Query\SelectInterface;

/**
 * Interface for query manager services.
 */
interface QueryManagerInterface {

  /**
   * Get translated and untranslated terms.
   *
   * @see QueryManagerInterface::alterTermTreeQuery()
   */
  const THBL_MODE_TREE_ALL = 1;

  /**
   * Get translated terms only.
   *
   * @see QueryManagerInterface::alterTermTreeQuery()
   */
  const THBL_MODE_TREE_TRANSLATED_ONLY = 2;

  /**
   * Tag to be added to queries altered by the query manager.
   */
  const THBL_TAG_QUERY_TRANSLATABLE = 'thbl_translatable';

  /**
   * Make the query generated by TermStorage::loadTree() aware of the language.
   *
   * @param Drupal\Core\Database\Query\SelectInterface $query
   *   The query to alter.
   * @param string $langcode
   *   Langcode to filter the query. If set to NULL the current content language
   *   is used (Drupal\Core\Language\LanguageInterface::TYPE_CONTENT).
   * @param int $mode
   *   (optional) Defines whether the query should return translated terms only
   *   or all terms including untranslated.
   *   One of:
   *     - QueryManagerInterface::THBL_MODE_TREE_ALL
   *     - QueryManagerInterface::THBL_MODE_TREE_TRANSLATED_ONLY
   *   Defaults to QueryManagerInterface::THBL_MODE_TREE_ALL.
   *
   * @see \Drupal\taxonomy\TermStorage::loadTree()
   */
  public function alterTermTreeQuery(SelectInterface &$query, $langcode = NULL, $mode = QueryManagerInterface::THBL_MODE_TREE_ALL): void;

  /**
   * Get the vocabulary identifier a term query is filtered by.
   *
   * @param Drupal\Core\Database\Query\SelectInterface $query
   *   The query filtered by a single vocabulary.
   *
   * @return string
   *   The vocabulary identifier the query is filtered by.
   */
  public function getVocabularyFromQuery(SelectInterface $query): string;

}
